from .types import getEClassifier, eClassifiers
from .types import name, nsURI, nsPrefix, eClass
from .types import Type, VisualType, ImportType, CompositeType, PointerType, QuantityType, ParameterType, StateVariableType, DynamicsType, ArgumentType, ExpressionType, HTMLType, TextType, URLType, PointType, ArrayType, CompositeVisualType, ConnectionType, SimpleType, ImageType
from . import types
from .. import model


__all__ = ['Type', 'VisualType', 'ImportType', 'CompositeType', 'PointerType', 'QuantityType', 'ParameterType', 'StateVariableType', 'DynamicsType',
           'ArgumentType', 'ExpressionType', 'HTMLType', 'TextType', 'URLType', 'PointType', 'ArrayType', 'CompositeVisualType', 'ConnectionType', 'SimpleType', 'ImageType']

eSubpackages = []
eSuperPackage = model
types.eSubpackages = eSubpackages
types.eSuperPackage = eSuperPackage


# Manage all other EClassifiers (EEnum, EDatatypes...)
otherClassifiers = []
for classif in otherClassifiers:
    eClassifiers[classif.name] = classif
    classif.ePackage = eClass

for classif in eClassifiers.values():
    eClass.eClassifiers.append(classif.eClass)

for subpack in eSubpackages:
    eClass.eSubpackages.append(subpack.eClass)



from pygeppetto.model import MDTimeSeries
from .model import GeppettoModel
from .variables import Variable, TypeToValueMap
from .values import Cylinder, Sphere, Point, PhysicalQuantity, TimeSeries, Unit
from pyecore.resources import ResourceSet, URI
import os.path


class GeppettoModelFactory():

    def __init__(self):
        rset = ResourceSet()
        # Build the model URI
        model_uri = URI(os.path.join(os.path.dirname(__file__), '..',
                                     'ecore', 'GeppettoCommonLibrary.xmi'))
        resource = rset.get_resource(model_uri)  # We load the model
        self.geppetto_common_library = resource.contents[0]  # We get the root

    def createGeppettoModel(self, name):
        # We create a GeppettoModel instance and we add the common library to it
        geppetto_model = GeppettoModel(name=name, libraries=[self.geppetto_common_library])
        return geppetto_model

    def createCylinder(self, id, bottomRadius=1.0, topRadius=1.0,
                       position=None, distal=None):
        position = position or Point()
        distal = distal or Point()
        variable = Variable(id=id)
        variable.types.append(self.geppetto_common_library.types[8])
        cylinder = Cylinder(bottomRadius=bottomRadius, topRadius=topRadius)
        cylinder.distal = distal
        cylinder.position = position
        variable.initialValues.append(TypeToValueMap(self.geppetto_common_library.types[8], cylinder))
        return variable

    def createSphere(self, id, radius=1.0, position=None):
        position = position or Point()
        variable = Variable(id=id)
        variable.types.append(self.geppetto_common_library.types[8])
        sphere = Sphere(radius=radius, position=position)
        variable.initialValues.append(TypeToValueMap(self.geppetto_common_library.types[8], sphere))
        return variable

    def createTimeSeries(self, id, values, unit=None):
        if unit:
            unit = Unit(unit)
        ts = TimeSeries(value=values, unit=unit)
        return ts

    def createMDTimeSeries(self, id, values):
        MD_ts = MDTimeSeries(value=values)
        return MD_ts

    def createStateVariable(self, id, initialValue=None):
        initialValue = initialValue or PhysicalQuantity()
        variable = Variable(id=id)
        variable.types.append(self.geppetto_common_library.types[2])
        if initialValue is not None:
            variable.initialValues.append(TypeToValueMap(self.geppetto_common_library.types[2],initialValue))
        return variable



from .variables import getEClassifier, eClassifiers
from .variables import name, nsURI, nsPrefix, eClass
from .variables import Variable, TypeToValueMap
from . import variables
from .. import model

__all__ = ['Variable', 'TypeToValueMap']

eSubpackages = []
eSuperPackage = model
variables.eSubpackages = eSubpackages
variables.eSuperPackage = eSuperPackage


# Manage all other EClassifiers (EEnum, EDatatypes...)
otherClassifiers = []
for classif in otherClassifiers:
    eClassifiers[classif.name] = classif
    classif.ePackage = eClass

for classif in eClassifiers.values():
    eClass.eClassifiers.append(classif.eClass)

for subpack in eSubpackages:
    eClass.eSubpackages.append(subpack.eClass)




from .datasources import getEClassifier, eClassifiers
from .datasources import name, nsURI, nsPrefix, eClass
from .datasources import DataSource, DataSourceLibraryConfiguration, Query, ProcessQuery, SimpleQuery, CompoundQuery, CompoundRefQuery, QueryResults, RunnableQuery, AQueryResult, QueryResult, SerializableQueryResult, QueryMatchingCriteria, BooleanOperator


from . import datasources
from .. import model


__all__ = ['DataSource', 'DataSourceLibraryConfiguration', 'Query', 'ProcessQuery', 'SimpleQuery', 'CompoundQuery', 'CompoundRefQuery',
           'QueryResults', 'RunnableQuery', 'AQueryResult', 'QueryResult', 'SerializableQueryResult', 'QueryMatchingCriteria', 'BooleanOperator']

eSubpackages = []
eSuperPackage = model
datasources.eSubpackages = eSubpackages
datasources.eSuperPackage = eSuperPackage


otherClassifiers = [BooleanOperator]

for classif in otherClassifiers:
    eClassifiers[classif.name] = classif
    classif.ePackage = eClass

for classif in eClassifiers.values():
    eClass.eClassifiers.append(classif.eClass)

for subpack in eSubpackages:
    eClass.eSubpackages.append(subpack.eClass)



"""Definition of meta model 'model'."""
from functools import partial
import pyecore.ecore as Ecore
from pyecore.ecore import *

name = 'model'
nsURI = 'https://raw.githubusercontent.com/openworm/org.geppetto.model/development/src/main/resources/geppettoModel.ecore'
nsPrefix = 'gep'

eClass = EPackage(name=name, nsURI=nsURI, nsPrefix=nsPrefix)

eClassifiers = {}
getEClassifier = partial(Ecore.getEClassifier, searchspace=eClassifiers)
FileFormat = EEnum('FileFormat', literals=['ZIP', 'HDF5'])  # noqa


@EMetaclass
class GeppettoModel(EObject):
    """The root of every Geppetto model"""
    id = EAttribute(eType=EString)
    name = EAttribute(eType=EString)
    variables = EReference(upper=-1, containment=True)
    libraries = EReference(upper=-1, containment=True)
    tags = EReference(upper=-1, containment=True)
    dataSources = EReference(upper=-1, containment=True)
    queries = EReference(upper=-1, containment=True)

    def __init__(self, variables=None, libraries=None, tags=None, id=None, name=None, dataSources=None, queries=None, **kwargs):
        if kwargs:
            raise AttributeError('unexpected arguments: {}'.format(kwargs))

        super(GeppettoModel, self).__init__()
        if id is not None:
            self.id = id
        if name is not None:
            self.name = name
        if variables:
            self.variables.extend(variables)
        if libraries:
            self.libraries.extend(libraries)
        if tags:
            self.tags.extend(tags)
        if dataSources:
            self.dataSources.extend(dataSources)
        if queries:
            self.queries.extend(queries)


@EMetaclass
class LibraryManager(EObject):
    libraries = EReference(upper=-1, containment=True)

    def __init__(self, libraries=None, **kwargs):
        if kwargs:
            raise AttributeError('unexpected arguments: {}'.format(kwargs))

        super(LibraryManager, self).__init__()
        if libraries:
            self.libraries.extend(libraries)


@EMetaclass
class ExperimentState(EObject):
    experimentId = EAttribute(eType=ELong)
    projectId = EAttribute(eType=ELong)
    recordedVariables = EReference(upper=-1, containment=True)
    setParameters = EReference(upper=-1, containment=True)

    def __init__(self, recordedVariables=None, setParameters=None, experimentId=None, projectId=None, **kwargs):
        if kwargs:
            raise AttributeError('unexpected arguments: {}'.format(kwargs))

        super(ExperimentState, self).__init__()
        if experimentId is not None:
            self.experimentId = experimentId
        if projectId is not None:
            self.projectId = projectId
        if recordedVariables:
            self.recordedVariables.extend(recordedVariables)
        if setParameters:
            self.setParameters.extend(setParameters)


@EMetaclass
class VariableValue(EObject):
    pointer = EReference(containment=True)
    value = EReference(containment=True)

    def __init__(self, pointer=None, value=None, **kwargs):
        if kwargs:
            raise AttributeError('unexpected arguments: {}'.format(kwargs))

        super(VariableValue, self).__init__()
        if pointer is not None:
            self.pointer = pointer
        if value is not None:
            self.value = value


@EMetaclass
class DomainModel(EObject):
    domainModel = EAttribute(eType=EJavaObject)
    format = EReference(containment=True)

    def __init__(self, domainModel=None, format=None, **kwargs):
        if kwargs:
            raise AttributeError('unexpected arguments: {}'.format(kwargs))

        super(DomainModel, self).__init__()
        if domainModel is not None:
            self.domainModel = domainModel
        if format is not None:
            self.format = format


@EMetaclass
class ModelFormat(EObject):
    modelFormat = EAttribute(eType=EString)

    def __init__(self, modelFormat=None, **kwargs):
        if kwargs:
            raise AttributeError('unexpected arguments: {}'.format(kwargs))

        super(ModelFormat, self).__init__()
        if modelFormat is not None:
            self.modelFormat = modelFormat


@EMetaclass
class StringToStringMap(EObject):
    key = EAttribute(eType=EString)
    value = EAttribute(eType=EString)

    def __init__(self, key=None, value=None, **kwargs):
        if kwargs:
            raise AttributeError('unexpected arguments: {}'.format(kwargs))

        super(StringToStringMap, self).__init__()
        if key is not None:
            self.key = key
        if value is not None:
            self.value = value


@abstract
@EMetaclass
class ISynchable(EObject):
    synched = EAttribute(eType=EBoolean)

    def __init__(self, synched=None, **kwargs):
        if kwargs:
            raise AttributeError('unexpected arguments: {}'.format(kwargs))

        super(ISynchable, self).__init__()
        if synched is not None:
            self.synched = synched


@abstract
class Node(ISynchable):
    id = EAttribute(eType=EString)
    name = EAttribute(eType=EString)
    tags = EReference(upper=-1)

    def __init__(self, id=None, name=None, tags=None, **kwargs):
        super(Node, self).__init__(**kwargs)
        if id is not None:
            self.id = id
        if name is not None:
            self.name = name
        if tags:
            self.tags.extend(tags)
    def getPath(self):
        raise NotImplementedError('Operation getPath(...) is not yet implemented')


class Tag(ISynchable):
    name = EAttribute(eType=EString)
    tags = EReference(upper=-1, containment=True)

    def __init__(self, tags=None, name=None, **kwargs):
        super(Tag, self).__init__(**kwargs)
        if name is not None:
            self.name = name
        if tags:
            self.tags.extend(tags)


class ExternalDomainModel(DomainModel):
    fileFormat = EAttribute(eType=FileFormat)

    def __init__(self, fileFormat=None, **kwargs):
        super(ExternalDomainModel, self).__init__(**kwargs)
        if fileFormat is not None:
            self.fileFormat = fileFormat


class GeppettoLibrary(Node):
    types = EReference(upper=-1, containment=True)
    sharedTypes = EReference(upper=-1)

    def __init__(self, types=None, sharedTypes=None, **kwargs):
        super(GeppettoLibrary, self).__init__(**kwargs)
        if types:
            self.types.extend(types)
        if sharedTypes:
            self.sharedTypes.extend(sharedTypes)

    def getTypeById(self):
        raise NotImplementedError('operation getTypeById(...) not yet implemented')



from backports.functools_lru_cache import lru_cache
from chainmap import ChainMap
from pyecore.resources.json import JsonResource
from . import model as pygeppetto
from . import datasources
from . import types
from . import values
from . import variables


class GeppettoResource(JsonResource):
    packages = [pygeppetto.eClassifiers,
                datasources.eClassifiers,
                types.eClassifiers,
                values.eClassifiers,
                variables.eClassifiers]
    chain = ChainMap(*packages)

    def serialize_eclass(self, eclass):
        return eclass.name

    @lru_cache()
    def resolve_eclass(self, uri):
        return self.chain.get(uri)



from .values import getEClassifier, eClassifiers
from .values import name, nsURI, nsPrefix, eClass
from .values import Value, Composite, StringToValueMap, Quantity, PhysicalQuantity, Unit, TimeSeries, MetadataValue, Text, URL, HTML, Pointer, PointerElement, Point, Dynamics, FunctionPlot, Function, Argument, Expression, VisualValue, Collada, OBJ, Sphere, Cylinder, Particles, SkeletonAnimation, SkeletonTransformation, VisualGroupElement, VisualGroup, Connection, Connectivity, ArrayElement, ArrayValue, Image, ImageFormat, ImportValue, MDTimeSeries
from . import values
from .. import model


__all__ = ['Value', 'Composite', 'StringToValueMap', 'Quantity', 'PhysicalQuantity', 'Unit', 'TimeSeries', 'MDTimeSeries', 'MetadataValue', 'Text', 'URL', 'HTML', 'Pointer', 'PointerElement', 'Point', 'Dynamics', 'FunctionPlot', 'Function', 'Argument',
           'Expression', 'VisualValue', 'Collada', 'OBJ', 'Sphere', 'Cylinder', 'Particles', 'SkeletonAnimation', 'SkeletonTransformation', 'VisualGroupElement', 'VisualGroup', 'Connection', 'Connectivity', 'ArrayElement', 'ArrayValue', 'Image', 'ImageFormat', 'ImportValue']

eSubpackages = []
eSuperPackage = model
values.eSubpackages = eSubpackages
values.eSuperPackage = eSuperPackage


# Manage all other EClassifiers (EEnum, EDatatypes...)
otherClassifiers = [Connectivity, ImageFormat]
for classif in otherClassifiers:
    eClassifiers[classif.name] = classif
    classif.ePackage = eClass

for classif in eClassifiers.values():
    eClass.eClassifiers.append(classif.eClass)

for subpack in eSubpackages:
    eClass.eSubpackages.append(subpack.eClass)



"""Definition of meta model 'variables'."""
from functools import partial
import pyecore.ecore as Ecore
from pyecore.ecore import *
from ..model import ISynchable, Node

name = 'variables'
nsURI = 'https://raw.githubusercontent.com/openworm/org.geppetto.model/development/src/main/resources/geppettoModel.ecore#//variables'
nsPrefix = 'gep'

eClass = EPackage(name=name, nsURI=nsURI, nsPrefix=nsPrefix)

eClassifiers = {}
getEClassifier = partial(Ecore.getEClassifier, searchspace=eClassifiers)



@EMetaclass
class TypeToValueMap(EObject):
    key = EReference()
    value = EReference(containment=True)

    def __init__(self, key=None, value=None, **kwargs):
        if kwargs:
            raise AttributeError('unexpected arguments: {}'.format(kwargs))

        super(TypeToValueMap, self).__init__()
        if key is not None:
            self.key = key
        if value is not None:
            self.value = value


class Variable(Node):
    static = EAttribute(eType=EBoolean)
    anonymousTypes = EReference(upper=-1, containment=True)
    types = EReference(upper=-1)
    initialValues = EReference(upper=-1, containment=True)
    position = EReference(containment=True)

    def __init__(self, anonymousTypes=None, types=None, initialValues=None, static=None, position=None, **kwargs):
        super(Variable, self).__init__(**kwargs)
        if static is not None:
            self.static = static
        if anonymousTypes:
            self.anonymousTypes.extend(anonymousTypes)
        if types:
            self.types.extend(types)
        if initialValues:
            self.initialValues.extend(initialValues)
        if position is not None:
            self.position = position



"""Definition of meta model 'types'."""
from functools import partial
import pyecore.ecore as Ecore
from pyecore.ecore import *
from ..model import ISynchable, Node

name = 'types'
nsURI = 'https://raw.githubusercontent.com/openworm/org.geppetto.model/development/src/main/resources/geppettoModel.ecore#//types'
nsPrefix = 'gep'

eClass = EPackage(name=name, nsURI=nsURI, nsPrefix=nsPrefix)

eClassifiers = {}
getEClassifier = partial(Ecore.getEClassifier, searchspace=eClassifiers)


@abstract
class Type(Node):
    abstract = EAttribute(eType=EBoolean)
    superType = EReference(upper=-1)
    visualType = EReference()
    referencedVariables = EReference(upper=-1)
    domainModel = EReference()

    def __init__(self, superType=None, abstract=None, visualType=None, referencedVariables=None, domainModel=None, **kwargs):
        super(Type, self).__init__(**kwargs)
        if abstract is not None:
            self.abstract = abstract
        if superType:
            self.superType.extend(superType)
        if visualType is not None:
            self.visualType = visualType
        if referencedVariables:
            self.referencedVariables.extend(referencedVariables)
        if domainModel is not None:
            self.domainModel = domainModel

    def getDefaultValue(self):
        raise NotImplementedError('Operation getDefaultValue(...) is not yet implemented')

    def extendsType(self, type):
        raise NotImplementedError('Operation extendsType(...) is not yet implemented')


class VisualType(Type):
    defaultValue = EReference(containment=True)

    def __init__(self, defaultValue=None, **kwargs):
        super(VisualType, self).__init__(**kwargs)
        if defaultValue is not None:
            self.defaultValue = defaultValue


class ImportType(Type):
    url = EAttribute(eType=EString)
    referenceURL = EAttribute(eType=EString)
    modelInterpreterId = EAttribute(eType=EString)
    autoresolve = EAttribute(eType=EBoolean, default_value=True)

    def __init__(self, url=None, referenceURL=None, modelInterpreterId=None, autoresolve=None, **kwargs):
        super(ImportType, self).__init__(**kwargs)
        if url is not None:
            self.url = url
        if referenceURL is not None:
            self.referenceURL = referenceURL
        if modelInterpreterId is not None:
            self.modelInterpreterId = modelInterpreterId
        if autoresolve is not None:
            self.autoresolve = autoresolve


class CompositeType(Type):
    variables = EReference(upper=-1, containment=True)
    defaultValue = EReference(containment=True)

    def __init__(self, variables=None, defaultValue=None, **kwargs):
        super(CompositeType, self).__init__(**kwargs)
        if variables:
            self.variables.extend(variables)
        if defaultValue is not None:
            self.defaultValue = defaultValue


class PointerType(Type):
    defaultValue = EReference(containment=True)

    def __init__(self, defaultValue=None, **kwargs):
        super(PointerType, self).__init__(**kwargs)
        if defaultValue is not None:
            self.defaultValue = defaultValue


class QuantityType(Type):
    defaultValue = EReference(containment=True)

    def __init__(self, defaultValue=None, **kwargs):
        super(QuantityType, self).__init__(**kwargs)
        if defaultValue is not None:
            self.defaultValue = defaultValue


class ParameterType(Type):
    defaultValue = EReference(containment=True)

    def __init__(self, defaultValue=None, **kwargs):
        super(ParameterType, self).__init__(**kwargs)
        if defaultValue is not None:
            self.defaultValue = defaultValue


class StateVariableType(Type):
    defaultValue = EReference(containment=True)

    def __init__(self, defaultValue=None, **kwargs):
        super(StateVariableType, self).__init__(**kwargs)
        if defaultValue is not None:
            self.defaultValue = defaultValue


class DynamicsType(Type):
    defaultValue = EReference(containment=True)

    def __init__(self, defaultValue=None, **kwargs):
        super(DynamicsType, self).__init__(**kwargs)
        if defaultValue is not None:
            self.defaultValue = defaultValue


class ArgumentType(Type):
    defaultValue = EReference(containment=True)

    def __init__(self, defaultValue=None, **kwargs):
        super(ArgumentType, self).__init__(**kwargs)
        if defaultValue is not None:
            self.defaultValue = defaultValue


class ExpressionType(Type):
    defaultValue = EReference(containment=True)

    def __init__(self, defaultValue=None, **kwargs):
        super(ExpressionType, self).__init__(**kwargs)
        if defaultValue is not None:
            self.defaultValue = defaultValue


class HTMLType(Type):
    defaultValue = EReference(containment=True)

    def __init__(self, defaultValue=None, **kwargs):
        super(HTMLType, self).__init__(**kwargs)
        if defaultValue is not None:
            self.defaultValue = defaultValue


class TextType(Type):
    defaultValue = EReference(containment=True)

    def __init__(self, defaultValue=None, **kwargs):
        super(TextType, self).__init__(**kwargs)
        if defaultValue is not None:
            self.defaultValue = defaultValue


class URLType(Type):
    defaultValue = EReference(containment=True)

    def __init__(self, defaultValue=None, **kwargs):
        super(URLType, self).__init__(**kwargs)
        if defaultValue is not None:
            self.defaultValue = defaultValue


class PointType(Type):
    defaultValue = EReference(containment=True)

    def __init__(self, defaultValue=None, **kwargs):
        super(PointType, self).__init__(**kwargs)
        if defaultValue is not None:
            self.defaultValue = defaultValue


class ArrayType(Type):
    size = EAttribute(eType=EInt)
    arrayType = EReference()
    defaultValue = EReference(containment=True)

    def __init__(self, size=None, arrayType=None, defaultValue=None, **kwargs):
        super(ArrayType, self).__init__(**kwargs)
        if size is not None:
            self.size = size
        if arrayType is not None:
            self.arrayType = arrayType
        if defaultValue is not None:
            self.defaultValue = defaultValue


class ConnectionType(Type):
    variables = EReference(upper=-1, containment=True)
    defaultValue = EReference(containment=True)

    def __init__(self, variables=None, defaultValue=None, **kwargs):
        super(ConnectionType, self).__init__(**kwargs)
        if variables:
            self.variables.extend(variables)
        if defaultValue is not None:
            self.defaultValue = defaultValue


class SimpleType(Type):

    def __init__(self, **kwargs):
        super(SimpleType, self).__init__(**kwargs)


class ImageType(Type):
    defaultValue = EReference()

    def __init__(self, defaultValue=None, **kwargs):
        super(ImageType, self).__init__(**kwargs)
        if defaultValue is not None:
            self.defaultValue = defaultValue


class CompositeVisualType(VisualType):
    variables = EReference(upper=-1, containment=True)
    visualGroups = EReference(upper=-1, containment=True)

    def __init__(self, variables=None, visualGroups=None, **kwargs):
        super(CompositeVisualType, self).__init__(**kwargs)
        if variables:
            self.variables.extend(variables)
        if visualGroups:
            self.visualGroups.extend(visualGroups)



class ModelInterpreter():

    def __init__(self):
        pass

    def importType(self, url, typeName, library, commonLibraryAccess):
        pass

    def importValue(self, importValue):
        pass

    def downloadModel(self, pointer, format, aspectConfiguration):
        pass

    def getSupportedOutputs(self, pointer):
        pass

    def getName(self):
        pass

    def getDependentModels(self):
        pass


"""Definition of meta model 'datasources'."""
from functools import partial
import pyecore.ecore as Ecore
from pyecore.ecore import *
from ..model import ISynchable, Node


name = 'datasources'
nsURI = 'https://raw.githubusercontent.com/openworm/org.geppetto.model/development/src/main/resources/geppettoModel.ecore#//datasources'
nsPrefix = 'gep'

eClass = EPackage(name=name, nsURI=nsURI, nsPrefix=nsPrefix)

eClassifiers = {}
getEClassifier = partial(Ecore.getEClassifier, searchspace=eClassifiers)
BooleanOperator = EEnum('BooleanOperator', literals=['AND', 'NAND', 'OR'])  # noqa




@EMetaclass
class DataSourceLibraryConfiguration(EObject):
    modelInterpreterId = EAttribute(eType=EString)
    format = EAttribute(eType=EString)
    library = EReference()

    def __init__(self, library=None, modelInterpreterId=None, format=None, **kwargs):
        if kwargs:
            raise AttributeError('unexpected arguments: {}'.format(kwargs))

        super(DataSourceLibraryConfiguration, self).__init__()
        if modelInterpreterId is not None:
            self.modelInterpreterId = modelInterpreterId
        if format is not None:
            self.format = format
        if library is not None:
            self.library = library


@EMetaclass
class QueryResults(EObject):
    id = EAttribute(eType=EString)
    header = EAttribute(eType=EString, upper=-1)
    results = EReference(upper=-1, containment=True)

    def __init__(self, id=None, header=None, results=None, **kwargs):
        if kwargs:
            raise AttributeError('unexpected arguments: {}'.format(kwargs))

        super(QueryResults, self).__init__()
        if id is not None:
            self.id = id
        if header:
            self.header.extend(header)
        if results:
            self.results.extend(results)

    def getValue(self, field, row):
        raise NotImplementedError('Operation getValue(...) is not yet implemented')


@EMetaclass
class RunnableQuery(EObject):
    targetVariablePath = EAttribute(eType=EString)
    queryPath = EAttribute(eType=EString)
    booleanOperator = EAttribute(eType=BooleanOperator)

    def __init__(self, targetVariablePath=None, queryPath=None, booleanOperator=None, **kwargs):
        if kwargs:
            raise AttributeError('unexpected arguments: {}'.format(kwargs))

        super(RunnableQuery, self).__init__()
        if targetVariablePath is not None:
            self.targetVariablePath = targetVariablePath
        if queryPath is not None:
            self.queryPath = queryPath
        if booleanOperator is not None:
            self.booleanOperator = booleanOperator


@abstract
@EMetaclass
class AQueryResult(EObject):

    def __init__(self, **kwargs):
        if kwargs:
            raise AttributeError('unexpected arguments: {}'.format(kwargs))

        super(AQueryResult, self).__init__()


@EMetaclass
class QueryMatchingCriteria(EObject):
    type = EReference(upper=-1)

    def __init__(self, type=None, **kwargs):
        if kwargs:
            raise AttributeError('unexpected arguments: {}'.format(kwargs))

        super(QueryMatchingCriteria, self).__init__()
        if type:
            self.type.extend(type)


class QueryResult(AQueryResult):
    values = EAttribute(eType=EJavaObject, upper=-1)

    def __init__(self, values=None, **kwargs):
        super(QueryResult, self).__init__(**kwargs)
        if values:
            self.values.extend(values)


class SerializableQueryResult(AQueryResult):
    values = EAttribute(eType=EString, upper=-1)

    def __init__(self, values=None, **kwargs):
        super(SerializableQueryResult, self).__init__(**kwargs)
        if values:
            self.values.extend(values)


class DataSource(Node):
    dataSourceService = EAttribute(eType=EString)
    url = EAttribute(eType=EString)
    libraryConfigurations = EReference(upper=-1, containment=True)
    queries = EReference(upper=-1, containment=True)
    dependenciesLibrary = EReference(upper=-1)
    targetLibrary = EReference()
    fetchVariableQuery = EReference(containment=True)

    def __init__(self, dataSourceService=None, libraryConfigurations=None, url=None, queries=None, dependenciesLibrary=None, targetLibrary=None, fetchVariableQuery=None, **kwargs):
        super(DataSource, self).__init__(**kwargs)
        if dataSourceService is not None:
            self.dataSourceService = dataSourceService
        if url is not None:
            self.url = url
        if libraryConfigurations:
            self.libraryConfigurations.extend(libraryConfigurations)
        if queries:
            self.queries.extend(queries)
        if dependenciesLibrary:
            self.dependenciesLibrary.extend(dependenciesLibrary)
        if targetLibrary is not None:
            self.targetLibrary = targetLibrary
        if fetchVariableQuery is not None:
            self.fetchVariableQuery = fetchVariableQuery


@abstract
class Query(Node):
    description = EAttribute(eType=EString)
    runForCount = EAttribute(eType=EBoolean)
    matchingCriteria = EReference(upper=-1, containment=True)
    returnType = EReference()

    def __init__(self, description=None, matchingCriteria=None, runForCount=None, returnType=None, **kwargs):
        super(Query, self).__init__(**kwargs)
        if description is not None:
            self.description = description
        if runForCount is not None:
            self.runForCount = runForCount
        if matchingCriteria:
            self.matchingCriteria.extend(matchingCriteria)
        if returnType is not None:
            self.returnType = returnType


class ProcessQuery(Query):
    queryProcessorId = EAttribute(eType=EString)
    parameters = EReference(upper=-1, containment=True)

    def __init__(self, parameters=None, queryProcessorId=None, **kwargs):
        super(ProcessQuery, self).__init__(**kwargs)
        if queryProcessorId is not None:
            self.queryProcessorId = queryProcessorId
        if parameters:
            self.parameters.extend(parameters)


class SimpleQuery(Query):
    query = EAttribute(eType=EString)
    countQuery = EAttribute(eType=EString)

    def __init__(self, query=None, countQuery=None, **kwargs):
        super(SimpleQuery, self).__init__(**kwargs)
        if query is not None:
            self.query = query
        if countQuery is not None:
            self.countQuery = countQuery


class CompoundQuery(Query):
    """Compound queries allow creating composite queries which chain together multiple queries. The results of a query in the chain will be fed to the subsequent query."""
    queryChain = EReference(upper=-1, containment=True)

    def __init__(self, queryChain=None, **kwargs):
        super(CompoundQuery, self).__init__(**kwargs)
        if queryChain:
            self.queryChain.extend(queryChain)


class CompoundRefQuery(Query):
    """Compound ref queries make it possible to reference queries from any datasource"""
    queryChain = EReference(upper=-1)

    def __init__(self, queryChain=None, **kwargs):
        super(CompoundRefQuery, self).__init__(**kwargs)
        if queryChain:
            self.queryChain.extend(queryChain)



try:
    from functools import lru_cache
except ImportError:
    from backports.functools_lru_cache import lru_cache
try:
    from collections import ChainMap
except ImportError:
    from chainmap import ChainMap
from pyecore.resources.json import JsonResource
from . import eClassifiers, datasources, types, values, variables


class GeppettoResource(JsonResource):
    packages = [eClassifiers,
                datasources.eClassifiers,
                types.eClassifiers,
                values.eClassifiers,
                variables.eClassifiers]
    chain = ChainMap(*packages)

    def serialize_eclass(self, eclass):
        return eclass.name

    @lru_cache()
    def resolve_eclass(self, uri):
        return self.chain.get(uri)



"""Definition of meta model 'values'."""
from functools import partial
import pyecore.ecore as Ecore
from pyecore.ecore import *
from ..model import ISynchable
from ..model import Node

name = 'values'
nsURI = 'https://raw.githubusercontent.com/openworm/org.geppetto.model/development/src/main/resources/geppettoModel.ecore#//values'
nsPrefix = 'gep'

eClass = EPackage(name=name, nsURI=nsURI, nsPrefix=nsPrefix)

eClassifiers = {}
getEClassifier = partial(Ecore.getEClassifier, searchspace=eClassifiers)
Connectivity = EEnum('Connectivity', literals=['DIRECTIONAL', 'BIDIRECTIONAL', 'NON_DIRECTIONAL'])  # noqa

ImageFormat = EEnum('ImageFormat', literals=[
                    'PNG', 'JPEG', 'IIP', 'DCM', 'NIFTI', 'TIFF', 'DZI', 'GOOGLE_MAP'])



@EMetaclass
class StringToValueMap(EObject):
    key = EAttribute(eType=EString)
    value = EReference(containment=True)

    def __init__(self, key=None, value=None, **kwargs):
        if kwargs:
            raise AttributeError('unexpected arguments: {}'.format(kwargs))

        super(StringToValueMap, self).__init__()
        if key is not None:
            self.key = key
        if value is not None:
            self.value = value


@EMetaclass
class PointerElement(EObject):
    index = EAttribute(eType=EInteger)
    variable = EReference()
    type = EReference()

    def __init__(self, variable=None, type=None, index=None, **kwargs):
        if kwargs:
            raise AttributeError('unexpected arguments: {}'.format(kwargs))

        super(PointerElement, self).__init__()
        if index is not None:
            self.index = index
        if variable is not None:
            self.variable = variable
        if type is not None:
            self.type = type


@EMetaclass
class FunctionPlot(EObject):
    title = EAttribute(eType=EString)
    xAxisLabel = EAttribute(eType=EString)
    yAxisLabel = EAttribute(eType=EString)
    initialValue = EAttribute(eType=EDouble)
    finalValue = EAttribute(eType=EDouble)
    stepValue = EAttribute(eType=EDouble)

    def __init__(self, title=None, xAxisLabel=None, yAxisLabel=None, initialValue=None, finalValue=None, stepValue=None, **kwargs):
        if kwargs:
            raise AttributeError('unexpected arguments: {}'.format(kwargs))

        super(FunctionPlot, self).__init__()
        if title is not None:
            self.title = title
        if xAxisLabel is not None:
            self.xAxisLabel = xAxisLabel
        if yAxisLabel is not None:
            self.yAxisLabel = yAxisLabel
        if initialValue is not None:
            self.initialValue = initialValue
        if finalValue is not None:
            self.finalValue = finalValue
        if stepValue is not None:
            self.stepValue = stepValue


@EMetaclass
class SkeletonTransformation(EObject):
    skeletonTransformation = EAttribute(eType=EDouble, upper=-1)

    def __init__(self, skeletonTransformation=None, **kwargs):
        if kwargs:
            raise AttributeError('unexpected arguments: {}'.format(kwargs))

        super(SkeletonTransformation, self).__init__()
        if skeletonTransformation:
            self.skeletonTransformation.extend(skeletonTransformation)


@abstract
class Value(ISynchable):

    def __init__(self, **kwargs):
        super(Value, self).__init__(**kwargs)


class Composite(Value):
    value = EReference(upper=-1, containment=True)

    def __init__(self, value=None, **kwargs):
        super(Composite, self).__init__(**kwargs)
        if value:
            self.value.extend(value)


class Quantity(Value):
    scalingFactor = EAttribute(eType=EInt)
    value = EAttribute(eType=EDouble)

    def __init__(self, scalingFactor=None, value=None, **kwargs):
        super(Quantity, self).__init__(**kwargs)
        if scalingFactor is not None:
            self.scalingFactor = scalingFactor
        if value is not None:
            self.value = value


class Unit(Value):
    unit = EAttribute(eType=EString)

    def __init__(self, unit=None, **kwargs):
        super(Unit, self).__init__(**kwargs)
        if unit is not None:
            self.unit = unit


class TimeSeries(Value):
    scalingFactor = EAttribute(eType=EInt)
    value = EAttribute(eType=EDouble, upper=-1)
    unit = EReference(containment=True)

    def __init__(self, unit=None, scalingFactor=None, value=None, **kwargs):
        super(TimeSeries, self).__init__(**kwargs)
        if scalingFactor is not None:
            self.scalingFactor = scalingFactor
        if value:
            self.value.extend(value)
        if unit is not None:
            self.unit = unit


class MDTimeSeries(Value):
    value = EReference(containment=True, upper=-1)

    def __init__(self, value=None, **kwargs):
        super(MDTimeSeries, self).__init__(**kwargs)
        if value is not None:
            self.value.extend(value)


@abstract
class MetadataValue(Value):

    def __init__(self, **kwargs):
        super(MetadataValue, self).__init__(**kwargs)


class Pointer(Value):
    path = EAttribute(eType=EString)
    elements = EReference(upper=-1, containment=True)
    point = EReference(containment=True)

    def __init__(self, elements=None, point=None, path=None, **kwargs):
        super(Pointer, self).__init__(**kwargs)
        if path is not None:
            self.path = path
        if elements:
            self.elements.extend(elements)
        if point is not None:
            self.point = point
    def getInstancePath(self):
        raise NotImplementedError('Operation getInstancePath(...) is not yet implemented')


class Point(Value):
    x = EAttribute(eType=EDouble)
    y = EAttribute(eType=EDouble)
    z = EAttribute(eType=EDouble)

    def __init__(self, x=None, y=None, z=None, **kwargs):
        super(Point, self).__init__(**kwargs)
        if x is not None:
            self.x = x
        if y is not None:
            self.y = y
        if z is not None:
            self.z = z


class Dynamics(Value):
    initialCondition = EReference(containment=True)
    dynamics = EReference(containment=True)

    def __init__(self, initialCondition=None, dynamics=None, **kwargs):
        super(Dynamics, self).__init__(**kwargs)
        if initialCondition is not None:
            self.initialCondition = initialCondition
        if dynamics is not None:
            self.dynamics = dynamics


class Function(Value):
    arguments = EReference(upper=-1, containment=True)
    expression = EReference(containment=True)
    functionPlot = EReference(containment=True)

    def __init__(self, arguments=None, expression=None, functionPlot=None, **kwargs):
        super(Function, self).__init__(**kwargs)
        if arguments:
            self.arguments.extend(arguments)
        if expression is not None:
            self.expression = expression
        if functionPlot is not None:
            self.functionPlot = functionPlot


class Argument(Value):
    argument = EAttribute(eType=EString)

    def __init__(self, argument=None, **kwargs):
        super(Argument, self).__init__(**kwargs)
        if argument is not None:
            self.argument = argument


class Expression(Value):
    expression = EAttribute(eType=EString)

    def __init__(self, expression=None, **kwargs):
        super(Expression, self).__init__(**kwargs)
        if expression is not None:
            self.expression = expression


@abstract
class VisualValue(Value):
    groupElements = EReference(upper=-1)
    position = EReference(containment=True)

    def __init__(self, groupElements=None, position=None, **kwargs):
        super(VisualValue, self).__init__(**kwargs)
        if groupElements:
            self.groupElements.extend(groupElements)
        if position is not None:
            self.position = position


class Particles(Value):
    particles = EReference(containment=True, upper=-1)

    def __init__(self, particles=None, **kwargs):
        super(Particles, self).__init__(**kwargs)
        if particles:
            self.particles.extend(particles)


class VisualGroupElement(Node):
    defaultColor = EAttribute(eType=EString)
    parameter = EReference(containment=True)

    def __init__(self, defaultColor=None, parameter=None, **kwargs):
        super(VisualGroupElement, self).__init__(**kwargs)
        if defaultColor is not None:
            self.defaultColor = defaultColor
        if parameter is not None:
            self.parameter = parameter


class VisualGroup(Node):
    lowSpectrumColor = EAttribute(eType=EString)
    highSpectrumColor = EAttribute(eType=EString)
    type = EAttribute(eType=EString)
    visualGroupElements = EReference(upper=-1, containment=True)

    def __init__(self, lowSpectrumColor=None, highSpectrumColor=None, type=None, visualGroupElements=None, **kwargs):
        super(VisualGroup, self).__init__(**kwargs)
        if lowSpectrumColor is not None:
            self.lowSpectrumColor = lowSpectrumColor
        if highSpectrumColor is not None:
            self.highSpectrumColor = highSpectrumColor
        if type is not None:
            self.type = type
        if visualGroupElements:
            self.visualGroupElements.extend(visualGroupElements)


class Connection(Value):
    connectivity = EAttribute(eType=Connectivity)
    a = EReference(containment=True)
    b = EReference(containment=True)

    def __init__(self, a=None, b=None, connectivity=None, **kwargs):
        super(Connection, self).__init__(**kwargs)
        if connectivity is not None:
            self.connectivity = connectivity
        if a is not None:
            self.a = a
        if b is not None:
            self.b = b


class ArrayElement(Value):
    index = EAttribute(eType=EInt)
    position = EReference(containment=True)
    initialValue = EReference(containment=True)

    def __init__(self, index=None, position=None, initialValue=None, **kwargs):
        super(ArrayElement, self).__init__(**kwargs)
        if index is not None:
            self.index = index
        if position is not None:
            self.position = position
        if initialValue is not None:
            self.initialValue = initialValue


class ArrayValue(Value):
    elements = EReference(upper=-1, containment=True)

    def __init__(self, elements=None, **kwargs):
        super(ArrayValue, self).__init__(**kwargs)
        if elements:
            self.elements.extend(elements)


class Image(Value):
    data = EAttribute(eType=EString)
    name = EAttribute(eType=EString)
    reference = EAttribute(eType=EString)
    format = EAttribute(eType=ImageFormat)

    def __init__(self, data=None, name=None, reference=None, format=None, **kwargs):
        super(Image, self).__init__(**kwargs)
        if data is not None:
            self.data = data
        if name is not None:
            self.name = name
        if reference is not None:
            self.reference = reference
        if format is not None:
            self.format = format


class ImportValue(Value):
    modelInterpreterId = EAttribute(eType=EString)

    def __init__(self, modelInterpreterId=None, **kwargs):
        super(ImportValue, self).__init__(**kwargs)
        if modelInterpreterId is not None:
            self.modelInterpreterId = modelInterpreterId


class PhysicalQuantity(Quantity):
    unit = EReference(containment=True)

    def __init__(self, unit=None, **kwargs):
        super(PhysicalQuantity, self).__init__(**kwargs)
        if unit is not None:
            self.unit = unit


class Text(MetadataValue):
    text = EAttribute(eType=EString)

    def __init__(self, text=None, **kwargs):
        super(Text, self).__init__(**kwargs)
        if text is not None:
            self.text = text


class URL(MetadataValue):
    url = EAttribute(eType=EString)

    def __init__(self, url=None, **kwargs):
        super(URL, self).__init__(**kwargs)
        if url is not None:
            self.url = url


class HTML(MetadataValue):
    html = EAttribute(eType=EString)

    def __init__(self, html=None, **kwargs):
        super(HTML, self).__init__(**kwargs)
        if html is not None:
            self.html = html


class Collada(VisualValue):
    collada = EAttribute(eType=EString)

    def __init__(self, collada=None, **kwargs):
        super(Collada, self).__init__(**kwargs)
        if collada is not None:
            self.collada = collada


class OBJ(VisualValue):
    obj = EAttribute(eType=EString)

    def __init__(self, obj=None, **kwargs):
        super(OBJ, self).__init__(**kwargs)
        if obj is not None:
            self.obj = obj


class Sphere(VisualValue):
    radius = EAttribute(eType=EDouble)

    def __init__(self, radius=None, **kwargs):
        super(Sphere, self).__init__(**kwargs)
        if radius is not None:
            self.radius = radius


class Cylinder(VisualValue):
    bottomRadius = EAttribute(eType=EDouble)
    topRadius = EAttribute(eType=EDouble)
    height = EAttribute(eType=EDouble)
    distal = EReference(containment=True)

    def __init__(self, bottomRadius=None, topRadius=None, height=None, distal=None, **kwargs):
        super(Cylinder, self).__init__(**kwargs)
        if bottomRadius is not None:
            self.bottomRadius = bottomRadius
        if topRadius is not None:
            self.topRadius = topRadius
        if height is not None:
            self.height = height
        if distal is not None:
            self.distal = distal


class SkeletonAnimation(VisualValue):
    skeletonTransformationSeries = EReference(upper=-1)

    def __init__(self, skeletonTransformationSeries=None, **kwargs):
        super(SkeletonAnimation, self).__init__(**kwargs)
        if skeletonTransformationSeries:
            self.skeletonTransformationSeries.extend(skeletonTransformationSeries)




from .utils.bytesuri import BytesURI
from .geppetto_resource import GeppettoResource
from pyecore.resources import ResourceSet
from pyecore.resources.json import JsonOptions


class GeppettoModelSerializer():

    def serialize(self, geppetto_model):
        #we now create a resource to save the geppetto model and serialize it to a JSON string
        rset = ResourceSet()
        uri = BytesURI('geppetto_model.json')
        rset.resource_factory['*'] = lambda uri: GeppettoResource(uri, indent=2)
        resource = rset.create_resource(uri)
        resource.append(geppetto_model)

        #TODO: We need to apply a visitor and set all the serialised objects to synched
        resource.save(options={JsonOptions.SERIALIZE_DEFAULT_VALUES: True})
        return uri.getvalue()

    def deserialize(self, serialized_geppetto_model):
        rset = ResourceSet()
        uri = BytesURI('geppetto_model.json', text=serialized_geppetto_model)
        rset.resource_factory['*'] = lambda uri: GeppettoResource(uri, indent=2)
        resource = rset.get_resource(uri)
        geppettomodel = resource.contents[0]
        return geppettomodel